/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Declares that the {@code Instance} region of the object referenced by this
 * field is to be mapped or "aggregated" into a particular named region of the
 * object that contains the field.
 * <p>
 * It is a modeling error if the field is not also annotated with {@link Unique}.
 * It is a modeling error if the field is also annotated with {@link Aggregate}.
 * 
 * <h3>Examples:</h3>
 * 
 * The below class defines a region, called {@code ObserverRegion}, that
 * includes the integer {@code notifyCounter} and the state referenced by the
 * {@code observers} field (i.e., the internals of the {@code HashSet} object).
 * 
 * <pre>
 * &#064;Region(&quot;private ObserverRegion&quot;)
 * class Observer {
 *   ...
 *   &#064;InRegion(&quot;ObserverRegion&quot;)
 *   private int notifyCounter = 0;
 *   ...
 *   &#064;Unique
 *   &#064;AggregateInRegion(&quot;ObserverRegion&quot;)
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 * }
 * </pre>
 * 
 * @see Aggregate
 * @see Unique
 */
@Documented
@Target(ElementType.FIELD)
public @interface AggregateInRegion {
	/**
	 * The value of this attribute indicates the region that the {@code
	 * Instance} region of <em>the object referenced by this field</em> is to be
	 * mapped or "aggregated" into. The value of this attribute must conform to
	 * the following grammar (in <a
	 * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
	 * Form</a>):
	 * 
	 * <pre>
	 * value = regionSpecification
	 * 
	 * regionSpecification = simpleRegionSpecificaion / qualifiedRegionName
	 * 
	 * simpleRegionSpecification = IDENTIFIER / &quot;[&quot; &quot;]&quot;                ; Region of the class being annotated
	 * 
	 * qualifedRegionName =  IDENTIFIER *(&quot;.&quot; IDENTIFIER) : IDENTIFER  ; Static region from the named, optionally qualified, class
	 * 
	 * IDENTIFIER = Legal Java Identifier
	 * </pre>
	 */
	String value();
}
