/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Marker annotation that directs the analysis to assume that the field or
 * parameter is {@code final}, even if the field really is not.
 * <p>
 * This annotation is trusted, i.e., it is <em>not verified by analysis</em>.
 * Its only valid use is to annotate a field that is used as a lock where, for
 * some reason, the field can't be declared to be {@code final}. Perhaps due to
 * a (questionable) protocol forced upon the program by a library or framework.
 * <p>
 * Because this annotation is trusted it is not recommended for use. Changing
 * your code such that the lock can be declared to be {@code final} is nearly
 * always the best approach.
 * 
 * <h3>Examples:</h3>
 * 
 * The below example, which is a highly questionable protocol, sets the lock
 * once after object construction and uses {@link AssumeFinal} to state this
 * policy.
 * 
 * <pre>
 * &#064;Region(&quot;private BadFinalRegion&quot;)
 * &#064;RegionLock(&quot;BadFinalLock is lock protects BadFinalRegion&quot;)
 * public class BadFinal {
 *   ...
 *   &#064;AssumeFinal
 *   private Object lock;
 * 
 *   // Called only once at startup
 *   public void setLock(Object value) {
 *     lock = value;
 *   }
 *   ...
 *   &#064;InRegion(&quot;BadFinalRegion&quot;)
 *   private int x, y;
 * 
 *   public void tick() {
 *     synchronized (lock) {
 *       x++;
 *       y++;
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * A better implementation is to set the lock via a constructor and avoid
 * the assumption about the lock reference entirely. In the example below, the
 * code is changed so that the lock field can be declared to be {@code final}.
 * 
 * <pre>
 * &#064;Region(&quot;private GoodFinalRegion&quot;)
 * &#064;RegionLock(&quot;GoodFinalLock is lock protects GoodFinalRegion&quot;)
 * public class GoodFinal {
 *   ...
 *   private final Object lock;
 * 
 *   public GoodFinal(Object value) {
 *     lock = value;
 *   }
 *   ...
 *   &#064;InRegion(&quot;GoodFinalRegion&quot;)
 *   private int x, y;
 * 
 *   public void tick() {
 *     synchronized (lock) {
 *       x++;
 *       y++;
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * @see RegionLock
 */
@Documented
@Target( { ElementType.FIELD, ElementType.PARAMETER })
public @interface AssumeFinal {
	// Marker annotation
}