/*
 * Copyright (c) 2005 Brian Goetz and Tim Peierls
 * Released under the Creative Commons Attribution License
 *   (http://creativecommons.org/licenses/by/2.5)
 * Official home: http://www.jcip.net
 *
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 * 
 * 
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.*;

/**
 * The class to which this annotation is applied is immutable. This means that
 * its state cannot be seen to change by callers, which implies that
 * <ul>
 * <li>all public fields are final,</li>
 * <li>all public final reference fields refer to other immutable objects, and</li>
 * <li>constructors and methods do not publish references to any internal state
 * which is potentially mutable by the implementation.</li>
 * </ul>
 * Immutable objects may still have internal mutable state for purposes of
 * performance optimization; some state variables may be lazily computed, so
 * long as they are computed from immutable state and that callers cannot tell
 * the difference.
 * <p>
 * Immutable objects are inherently thread-safe; they may be passed between
 * threads or published without synchronization.
 * <p>
 * This annotation is trusted, i.e., it is <em>not verified by analysis</em>.
 * Presently its use is for documentation and to quiet warnings similar to
 * {@link ThreadSafe}. It is intended that this annotation will be verified in
 * the future.
 * <p>
 * <i>Implementation note:</i> This annotation is derived from
 * <code>&#064;Immutable</code> proposed by Brian Goetz and Tim Peierls in the
 * book <i>Java Concurrency in Practice</i> (Addison-Wesley 2006) we have simply
 * adapted it to have semantics as a promise. Further, the annotation in {@code
 * net.jcip.annotations} may be used instead of this one with the same tool
 * behavior. One difference between the two annotations is that the one in
 * {@code net.jcip.annotations} has retention policy of
 * {@link java.lang.annotation.RetentionPolicy#RUNTIME} while the one in {@code
 * com.surelogic} defaults to {@link java.lang.annotation.RetentionPolicy#CLASS}.
 * 
 * <h3>Examples:</h3>
 * The immutable {@code Point} class below is considered thread-safe.
 * 
 * <pre>
 * &#064;Immutable
 * public class Point {
 * 
 *   final int f_x;
 *   final int f_y;
 * 
 *   public Point(int x, int y) {
 *     f_x = x;
 *     f_y = y;
 *   }
 * 
 *   public int getX() {
 *     return f_x;
 *   }
 * 
 *   public int getY() {
 *     return f_y;
 *   }
 * }
 * </pre>
 * 
 * @see ThreadSafe
 */
@Documented
@Target(ElementType.TYPE)
public @interface Immutable {
	// Marker annotation
}
