/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Declare a new region lock in the annotated class. Creates a new named lock
 * that associates a particular lock object with a region of the class. The
 * region may only be accessed when the lock is held.
 * <p>
 * To declare more than one lock for a class use the {@link RegionLocks}
 * annotation. It is a modeling error for a class to have both a
 * {@link RegionLocks} and a {@link RegionLock} annotation.
 * <p>
 * The named lock is a Java object. If the object's type implements {@code
 * java.util.concurrent.locks.Lock} then the lock object must be used according
 * to the protocol of the {@code Lock} interface. Otherwise, the object must be
 * used as a Java intrinsic lock, i.e., with {@code synchronized} blocks.
 * 
 * <h3>Examples:</h3>
 * 
 * A locking policy, named {@code SimpleLock}, that indicates that synchronizing
 * on the instance protects the all of the instance's state.
 * 
 * <pre>
 * &#064;RegionLock(&quot;SimpleLock is this protects Instance&quot;)
 * class Simple { ... }
 * </pre>
 * 
 * A locking policy, named {@code ObserverLock}, that indicates that
 * synchronizing on the field {@code observerLock} (which must be declared to
 * be {@code final}) protects the contents of the set {@code observers}.
 * 
 * <pre>
 * &#064;Region(&quot;private ObserverRegion&quot;)
 * &#064;RegionLock(&quot;ObserverLock is observerLock protects ObserverRegion&quot;)
 * class Observer {
 *   private final Object observerLock = new Object();
 *   ...
 *   &#064;Unique
 *   &#064;Aggregate(&quot;Instance into ObserverRegion&quot;)
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 *   private void add(Callback c) {
 *     synchronized (observerLock) { observers.add(c); }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * A locking policy, named {@code StateLock}, that indicates that locking on the
 * {@code java.util.concurrent.Lock} {@code stateLock} protects the two {@code
 * long} fields use to represent the position of the object.
 * 
 * <pre>
 * &#064;Region(&quot;private AircraftState&quot;)
 * &#064;RegionLock(&quot;StateLock is stateLock protects AircraftState&quot;)
 * public class Aircraft {
 *   private final Lock stateLock = new ReentrantLock();
 *   ...
 *   &#064;InRegion(&quot;AircraftState&quot;)
 *   private long x, y;
 *   ...
 *   public void setPosition(long x, long y) {
 *     stateLock.lock();
 *     try {
 *       this.x = x;
 *       this.y = y;
 *     } finally {
 *       stateLock.unlock();
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * A {@link SingleThreaded} annotation together with a {@link Borrowed} annotation is often needed
 * on a constructor because if the receiver is not leaked during object
 * construction then the state under construction will remain within the thread
 * that invoked {@code new}.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * public class Example {
 * 
 *   int x = 1;
 *   int y;
 * 
 *   &#064;SingleThreaded
 *   &#064;Borrowed(&quot;this&quot;)
 *   public Example(int y) {
 *     this.y = y;
 *   }
 *   ...
 * }
 * </pre>
 * 
 * The scoped promise {@link Promise} can be used if the constructor is
 * implicit. It has the ability to place promises on implicit and explicit
 * constructors.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * &#064;Promises( {
 *   &#064;Promise(&quot;&#064;SingleThreaded for new(**)&quot;),
 *   &#064;Promise(&quot;&#064;Borrowed(this) for new(**)&quot;)
 * })
 * public class Example {
 * 	int x = 1;
 * 	int y = 1;
 *  ...
 * }
 * </pre>
 * 
 * It is also possible to support the {@link SingleThreaded} claim with effects
 * ({@link Starts} and {@link RegionEffects}) as shown in the example code
 * below.
 * 
 * <pre>
 * &#064;RegionLock("Lock is this protects Instance")
 * public class Example {
 * 
 *   int x = 1;
 *   int y;
 * 
 *   &#064;SingleThreaded
 *   &#064;Starts("nothing")
 *   &#064;RegionEffects("none")
 *   public Example(int y) {
 *     this.y = y;
 *   }
 *   ...
 * }
 * </pre>
 * 
 * @see RegionLocks
 * @see PolicyLock
 */
@Documented
@Target(ElementType.TYPE)
public @interface RegionLock {
	/**
	 * The value of this attribute must conform to the following grammar (in <a
	 * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
	 * Form</a>):
	 * 
	 * <pre>
	 * value = IDENTIFIER &quot;is&quot; lockExpression &quot;protects&quot; regionName
	 * 
	 * lockExpression = simpleLockExpression / qualifiedLockExpression
	 * 
	 * simpleLockExpression = 
	 *   &quot;class&quot; /              ; the Class object referenced by the &quot;class&quot; pseudo-field of the annotated class
	 *   &quot;this&quot; /               ; the instance itself
	 *   &quot;this&quot; &quot;.&quot; IDENTIFIER  ; the object referenced by the named field
	 * 
	 * qualifiedLockExpression = 
	 *   namedType &quot;.&quot; &quot;CLASS&quot; /            ; the Class object referenced by the &quot;class&quot; pseudo-field of a named class
	 *   namedType &quot;.&quot; &quot;THIS&quot; /             ; a named enclosing instance
	 *   namedType &quot;.&quot; IDENTIFIER /         ; a named static field
	 *   namedType &quot;.&quot; THIS &quot;.&quot; IDENTIFIER  ; a named field of an enclosing instance
	 * 
	 * namedType = IDENTIFIER *(&quot;.&quot; IDENTIFIER)
	 * 
	 * regionName = IDENTIFIER
	 * 
	 * IDENTIFIER = Legal Java Identifier
	 * </pre>
	 */
	String value();
}
