/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Marker annotation that indicates that the object created by the constructor
 * is only operated on by the thread that invoked the constructor. This
 * knowledge allows fields of the newly created object to be accessed without
 * the normally required locking within the body of the constructor.
 * <p>
 * A {@link SingleThreaded} annotation must always be supported by another
 * promise. Presently there are two approaches to support this assertion. The
 * first is to support the assertion with a {@link Borrowed} annotation (an
 * aliasing claim). The constructor must promise not to alias the receiver
 * (i.e., the object under construction). The second is to support the assertion
 * with {@link Starts} annotation and a {@link RegionEffects} annotation (an
 * effects claim). The constructor must promise to start no threads and not to
 * write into any state outside the object under construction (it can read any
 * state).
 * 
 * <h3>Examples:</h3>
 * 
 * A {@link SingleThreaded} with a {@link Borrowed} annotation is often needed
 * on a constructor because if the receiver is not leaked during object
 * construction then the state under construction will remain within the thread
 * that invoked {@code new}.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * public class Example {
 * 
 *   int x = 1;
 *   int y;
 * 
 *   &#064;SingleThreaded
 *   &#064;Borrowed(&quot;this&quot;)
 *   public Example(int y) {
 *     this.y = y;
 *   }
 *   ...
 * }
 * </pre>
 * 
 * The scoped promise {@link Promise} can be used if the constructor is
 * implicit. It has the ability to place promises on implicit and explicit
 * constructors.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * &#064;Promises( {
 *   &#064;Promise(&quot;&#064;SingleThreaded for new(**)&quot;),
 *   &#064;Promise(&quot;&#064;Borrowed(this) for new(**)&quot;)
 * })
 * public class Example {
 * 	int x = 1;
 * 	int y = 1;
 *  ...
 * }
 * </pre>
 * 
 * A locking policy, named {@code StateLock}, that indicates that synchronizing
 * on the field {@code stateLock} (which must be declared to be {@code final})
 * protects the two {@code long} fields use to represent the position of the
 * object. The {@link SingleThreaded} annotation is used to allow the
 * constructor to assign the position of the object without holding the lock. In
 * this example the {@link SingleThreaded} annotation is supported by a
 * {@link Borrowed} annotation.
 * 
 * <pre>
 * &#064;Region(&quot;private AircraftState&quot;)
 * &#064;RegionLock(&quot;StateLock is stateLock protects AircraftState&quot;)
 * public class Aircraft {
 *   private final Object stateLock = new Object();
 *   ...
 *   &#064;InRegion(&quot;AircraftState&quot;)
 *   private long x, y;
 *   ...
 *   &#064;SingleThreaded
 *   &#064;Borrowed(&quot;this&quot;)
 *   public Aircraft(long x, long y) {
 *     this.x = x;
 *     this.y = y;
 *   }
 *   ...
 *   public void setPosition(long x, long y) {
 *     synchronized(stateLock)
 *       this.x = x;
 *       this.y = y;
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * It is also possible to support the {@link SingleThreaded} claim with effects
 * ({@link Starts} and {@link RegionEffects}) as shown in the example code
 * below.
 * 
 * <pre>
 * &#064;RegionLock("Lock is this protects Instance")
 * public class Example {
 * 
 *   int x = 1;
 *   int y;
 * 
 *   &#064;SingleThreaded
 *   &#064;Starts("nothing")
 *   &#064;RegionEffects("none")
 *   public Example(int y) {
 *     this.y = y;
 *   }
 *   ...
 * }
 * </pre>
 * 
 * The {@link RegionEffects} annotation can be more general than the one shown
 * in the example above. The effects annotation must be (or be more restrictive
 * than) <code>&#064;RegionEffects("reads All")</code>. This states that any
 * state can be read but only the object under construction can be written to
 * (for more information see {@link RegionEffects}). That said, the annotation
 * in the example above is adequate for most constructors.
 */
@Documented
@Target(ElementType.CONSTRUCTOR)
public @interface SingleThreaded {
	// Marker annotation
}
