/*
 * Copyright (c) 2005 Brian Goetz and Tim Peierls
 * Released under the Creative Commons Attribution License
 *   (http://creativecommons.org/licenses/by/2.5)
 * Official home: http://www.jcip.net
 *
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 * 
 * 
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * The class to which this annotation is applied is thread-safe. This means that
 * no sequences of accesses (reads and writes to public fields, calls to public
 * methods) may put the object into an invalid state, regardless of the
 * interleaving of those actions by the runtime, and without requiring any
 * additional synchronization or coordination on the part of the caller. Even if
 * one or more {@link RegionLock} models has been developed to document the
 * locking policy of a class this annotation can help to clarify that the
 * overall class is thread-safe.
 * <p>
 * This annotation does not imply anything about how the class is implemented.
 * Of course, it also does not imply that a sequence of calls to methods of this
 * class are atomic. It is a the responsibility of the caller to insure that
 * such call sequences execute atomically
 * <p>
 * This annotation is trusted, i.e., it is <em>not verified by analysis</em>.
 * Its use is for documentation and to quiet warnings that would otherwise be
 * raised when accessing an object through a field protected by a lock.
 * <p>
 * <i>Implementation note:</i> This annotation is derived from
 * <code>&#064;ThreadSafe</code> proposed by Brian Goetz and Tim Peierls in the
 * book <i>Java Concurrency in Practice</i> (Addison-Wesley 2006) we have simply
 * adapted it to have semantics as a promise. Further, the annotation in {@code
 * net.jcip.annotations} may be used instead of this one with the same tool
 * behavior. One difference between the two annotations is that the one in
 * {@code net.jcip.annotations} has retention policy of
 * {@link java.lang.annotation.RetentionPolicy#RUNTIME} while the one in {@code
 * com.surelogic} defaults to {@link java.lang.annotation.RetentionPolicy#CLASS}.
 * 
 * <h3>Examples:</h3>
 * 
 * The {@code Aircraft} class declares that it is thread-safe as part of its
 * lock policy documentation.
 * 
 * <pre>
 * &#064;ThreadSafe
 * &#064;Region(&quot;private AircraftState&quot;)
 * &#064;RegionLock(&quot;StateLock is stateLock protects AircraftState&quot;)
 * public class Aircraft {
 *   private final Lock stateLock = new ReentrantLock();
 *   ...
 *   &#064;InRegion(&quot;AircraftState&quot;)
 *   private long x, y;
 *   ...
 *   public void setPosition(long x, long y) {
 *     stateLock.lock();
 *     try {
 *       this.x = x;
 *       this.y = y;
 *     } finally {
 *       stateLock.unlock();
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * @see Immutable
 * @see NotThreadSafe
 * @see RegionLock
 */
@Documented
@Target(ElementType.TYPE)
public @interface ThreadSafe {
	// Marker annotation
}