/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Indicates that the parameter, receiver, return value, or field is a unique
 * reference to an object. Normally this indicates that the referenced object is
 * unalised. It is allowed, however, to pass a unique reference to a method as
 * an actual argument or the receiver if formal argument or receiver, respectively,
 * is {@link Borrowed}. That is, {@link Unique} values can be safely
 * passed to the parameter or used as the receiver with the guarantee that they
 * will still be unique when the method returns. Said another way we create a
 * temporary alias on the stack then ensure that it goes away.
 * <p>
 * It is a modeling error to annotate a method return value if the return type
 * is primitive (e.g., <code>&#64;Unique("return") public int getValue()</code>
 * would generate a modeling error). It is a modeling error to annotate a
 * parameter if the parameter's type is primitive (e.g.,
 * <code>public void setValue(&#64;Unique int value)</code> would generate a
 * modeling error).
 * 
 * <h3>Examples:</h3>
 * 
 * A class with a unique field. Note that the {@code getUniqueObject} ensures
 * that the returned reference is unique by creating a new object.
 * 
 * <pre>
 * public class UniqueField {
 *   &#064;Unique private Object o;
 * 
 *   public void setField(@Unique Object value) {
 *     o = value;
 *     process(o);
 *   }
 * 
 *   &#064;Unique(&quot;return&quot;)
 *   public Object getUniqueObject() {
 *     return new Object();
 *   }
 *   
 *   &#064;RegionEffects("reads value:Instance")
 *   public static void process(@Borrowed Object value) {
 *     // for future processing
 *   }
 * }
 * </pre>
 * 
 * The call to {@code process} is allowed because that method promises that the
 * alias to {@code o} created by the {@code value} parameter will be returned
 * once the method call completes. However, {@code process} must also promise
 * not to read state outside of its passed parameters. This is done with the
 * {@link RegionEffects} annotation on the {@code process} method declaration.
 * 
 * @see Borrowed
 */
@Documented
@Target( { ElementType.FIELD, ElementType.METHOD, ElementType.PARAMETER })
public @interface Unique {
	/**
	 * When annotating a method, this attribute is used to disambiguate whether
	 * the annotation refers to the method's receiver, the method's return
	 * value, or both. The value is comma separated list of tokens, and has the
	 * following set of legal values (ignoring white space issues):
	 * <ul>
	 * <li>{@code ""}
	 * <li>{@code "this"}
	 * <li>{@code "return"}
	 * <li>{@code "this, return"}
	 * <li>{@code "return, this"}
	 * </ul>
	 * 
	 * <p>
	 * The values are interpreted thusly
	 * <ul>
	 * <li>If the list contains the value {@code "this"}, it indicates the
	 * receiver is unique.
	 * <li>If the list contains the value {@code "return"}, or is if the list is
	 * of length zero, it indicates the return value is unique.
	 * <li>If the list contains both {@code "this"} and {@code "return"}, it
	 * indicates that both the receiver and the return value are unique.
	 * </ul>
	 * <p>
	 * This attribute is not used when annotating a parameter or a field: the
	 * attribute value must be the empty string in these cases.
	 * <p>
	 * The value of this attribute must conform to the following grammar (in <a
	 * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
	 * Form</a>):
	 * 
	 * <pre>
	 * value = [(&quot;this&quot; [&quot;,&quot; &quot;return&quot;]) / (&quot;return&quot; [&quot;,&quot; &quot;this&quot;])] ; See above comments
	 * </pre>
	 */
	String value() default "";
}
