/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Vouches for any any inconsistent analysis result within the scope of code
 * that the annotation appears on. This means that any inconsistent result will
 * be changed to a consistent result. Its use is for documentation and to quiet
 * overly conservative analysis results.
 * <p>
 * This annotation is trusted, i.e., it is <em>not verified by analysis</em>.
 * <p>
 * The annotation requires a brief programmer explanation for the vouch being
 * made.
 * 
 * <h3>Examples:</h3>
 * 
 * An {@code init} method is used to set set state, perhaps due to an API
 * restriction about using constructors, then {@code CentralControl} instances
 * are safely published. A {@link Vouch} is used to explain that the {@code
 * init} method needs to be an exception to the lock policy.
 * 
 * <pre>
 * &#064;Region("private ControlRegion")
 * &#064;InRegion("f_id into ControlRegion")
 * &#064;RegionLock("ControlLock is lock protects ControlRegion")
 * public class CentralControl {
 * 
 *   private final Object lock = new Object();
 *   private String f_id;
 * 
 *   &#064;Vouch("Instances are thread confined until after init(String) is called.")
 *   public void init(String id) {
 *     f_id = id;
 *   }
 * 
 *   public String getId() {
 *     synchronized (lock) {
 *       return f_id;
 *     }
 *   }
 * 
 *   public void setId(String value) {
 *     synchronized (lock) {
 *       f_id = value;
 *     }
 *   }
 * }
 * </pre>
 * 
 * A {@link Vouch} is used to explain that the {@code SmokeTest} class is test
 * code.
 * 
 * <pre>
 * &#064;Vouch("Test code that is intentionally inconsistent with models")
 * public class SmokeTest extends ... {
 *   ...
 * }
 * </pre>
 */
@Documented
@Target( { ElementType.TYPE, ElementType.METHOD, ElementType.CONSTRUCTOR })
public @interface Vouch {
  /**
   * A brief explanation of why the annotated code should have its inconsistent
   * assurance results suppressed.
   */
	public String value();
}
